# controller.py
# Walker M. White (wmw2)
# October 19, 2012
"""Controller class/module for temperature converter

This class represents the CONTROLLER.  The controller is responsible for 
sending messages between the view and the model. Therefore, it needs to 
have fields referencing the model and the view so that it can send each 
of them messages. In most cases (including this one), the controller is
also responsible for starting the program.  That is what the run()
method does.

This class has some features, such as callback functions, that we will
not introduce until towards the end of the course."""
from kivy.app import App
from kivy.config import Config

# Coordinate the other classes
import view
import model

class ConverterApp(App):
    _model = None # Model for converter
    _view = None  # View for converter
    
    # NO PROPERTIES (No one else needs to access fields)
    
    """Instances are a single converter application"""
    def build(self):
        """Read kivy file and perform layout"""
        Config.set('graphics', 'width', '500')
        Config.set('graphics', 'height', '200')
        
        # Initialize the model and the view
        self._view  = view.TemperaturePanel()
        self._model = model.Temperature()
        
        # Initialize the text field values.
        self.updateView()
        
        # Add callbacks to react to changes in input.
        self._view.fahrenheitField.bind(on_text_validate=self.updateFahrenheit)
        self._view.centigradeField.bind(on_text_validate=self.updateCentigrade)
        
        return self._view # Kivy needs view to start
    
    def updateView(self):
        """Update the view to be consistent with the model"""
        self._view.fahrenheitField.text = str(round(self._model.fahrenheit,4))
        self._view.centigradeField.text = str(round(self._model.centigrade,4))
        
    
    def updateFahrenheit(self,field):
        """Reacts to a change in the farenheit text field
        
        Precondition: field is the textinput field"""
        result = field.text
        value = 0.0
        try:
            # This might fail
            value = float(result)
        except:
            field.text = str(value)
        self._model.fahrenheit = value
        self.updateView()
        
    def updateCentigrade(self,field):
        """Reacts to a change in the centigrade text field
        
        Precondition: field is the textinput field"""
        result = field.text
        value = 0.0
        try:
            # This might fail
            value = float(result)
        except:
            field.text = str(value)
        self._model.centigrade = value
        self.updateView()


# Application Code
if __name__ in ('__android__', '__main__'):
    ConverterApp().run()