# model.py
# Walker M. White (wmw2)
# October 19, 2012
"""Model class/module for temperature converter.

This is the MODEL that holds our temperature values and converts
between them for us.  We separate this from the view so that we
can restore the old state whenever a user enters bad data.  If the
view took care of everything, we would not be able to undo bad
input.

At any given time, this model should be *consistent.  That is, 
the farenheit and the centigrade value should never get out of synch.
Therefore, whenever the farenheit value is changed, the centigrade
value must be changed immediately as well.  This is a prime example
of why you should NEVER make fields public, but use properties
(which can enforce invariants) instead.

In order to ensure consistency, we will only have one field: the 
farenheit field.  The centigrade property is computed on the fly 
whenever we need it.

Note that the model NEVER talks to the controller or the view.  The
model is an entirely passive object; the controller does all of the
talking.  This allows for maximum code reuse."""

class Temperature(object):
    """Instance represents a temperature value."""
    _fahrenheit = 32.0 # Temperature in farenheit
    
    @property
    def fahrenheit(self):
        """The temperature value in farenheit
        
        Invariant: Must be a float
        Must be equal 9*centigrade/5.0+32"""
        return self._fahrenheit
    
    @fahrenheit.setter
    def fahrenheit(self,value):
        assert type(value) == int or type(value) == float, `value`+' is not a number'
        self._fahrenheit = float(value) # Enforce the invariant
    
    @property
    def centigrade(self):
        """The temperature value in centigrade
        
        Invariant: Must be a float
        Must be equal 5*(farenheit-32)/9.0"""
        return 5*(self.fahrenheit-32)/9.0
    
    @centigrade.setter
    def centigrade(self,value):
        assert type(value) == int or type(value) == float, `value`+' is not a number'
        self._fahrenheit = 9*float(value)/5.0+32 # Enforce the invariant
    
    def __init__(self, fahrenheit=None,centigrade=None):
        """Constructor: Creates a temperature with the given farenheit or centigrade value.
        Preconditions: fahrenheit and centigrade are both either a number or None
                       Both farenheit and centrigrade cannot be none."""
        assert fahrenheit is None or centigrade is None, 'You can only have one temparature parameter'
        if not fahrenheit is None:
            self.fahrenheit = fahrenheit # Property takes care of other preconditions
        if not centigrade is None:
            self.centigrade = centigrade # Property takes care of other preconditions

    def __repr__(self):
        """Returns: Unambiguous tring representation of this temperature"""
        return str(self.__class__) + str(self)
 
    def __str__(self):
        """Returns: String representation of this temperature"""
        return str(self.fahrenheit) + ' F (' + str(self.centigrade) + ' C)'
    
    def __eq__(self,other):
        """Returns: True if other is an equal temperature."""
        if not instanceof(other,Temperature):
            return false
        # Compare floats by checking if the difference is small.
        return abs(other.fahrenheit-self.fahrenheit) < 1e-12

